@@ -18,7 +18,7 @@
 #define __NR_sys_vmsplice	309
 #endif
 
-#define nop		asm volatile ("diag 0,0,68" : : : "memory")
+#define nop		asm volatile("nop" : : : "memory")
 #define read_barrier()	asm volatile("bcr 15,0" : : : "memory")
 #define write_barrier()	asm volatile("bcr 15,0" : : : "memory")
 
@@ -26,10 +26,21 @@ static inline unsigned long long get_cpu_clock(void)
 {
 	unsigned long long clk;
 
+#ifdef CONFIG_S390_Z196_FACILITIES
+	/*
+	 * Fio needs monotonic (never lower), but not strict monotonic (never
+	 * the same) so store clock fast is enough.
+	 */
+	__asm__ __volatile__("stckf %0" : "=Q" (clk) : : "cc");
+#else
 	__asm__ __volatile__("stck %0" : "=Q" (clk) : : "cc");
-	return clk;
+#endif
+	return clk>>12;
 }
 
+#define ARCH_CPU_CLOCK_CYCLES_PER_USEC 1
+#define ARCH_HAVE_CPU_CLOCK
+
 #define ARCH_HAVE_INIT
 extern int tsc_reliable;
 static inline int arch_init(char *envp[])
@@ -38,6 +49,4 @@ static inline int arch_init(char *envp[])
 	return 0;
 }
 
-#define ARCH_HAVE_CPU_CLOCK
-
 #endif
