@@ -59,7 +59,7 @@ static int fio_mmap_file(struct thread_data *td, struct fio_file *f,
 err:
 	if (td->error && f->mmap_ptr)
 		munmap(f->mmap_ptr, length);
-		
+
 	return td->error;
 }
 
@@ -187,7 +187,7 @@ static int fio_mmapio_init(struct thread_data *td)
 	if ((td->o.rw_min_bs & page_mask) &&
 	    (o->odirect || o->fsync_blocks || o->fdatasync_blocks)) {
 		log_err("fio: mmap options dictate a minimum block size of "
-			"%lu bytes\n", page_size);
+			"%llu bytes\n", (unsigned long long) page_size);
 		return 1;
 	}
 
@@ -200,7 +200,7 @@ static int fio_mmapio_init(struct thread_data *td)
 			break;
 		shift++;
 	} while (1);
-		
+
 	mmap_map_mask = 1UL << shift;
 	return 0;
 }
